package com.fr.plugin;

import com.fr.decision.fun.impl.AbstractEmbedRequestFilterProvider;
import com.fr.json.JSONObject;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.utils.SocketIOUitls;
import com.fr.stable.StringUtils;
import com.fr.third.socketio.HandshakeData;
import com.fr.third.socketio.SocketIOClient;
import com.fr.third.socketio.SocketIOServer;
import io.netty.util.internal.PlatformDependent;

import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.*;

public class InitSocketFilter extends AbstractEmbedRequestFilterProvider {

    @Override
    public void filter(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) throws IOException, ServletException {

    }

    public static List<SocketIOClient> ALL_CLIENT = Collections.synchronizedList(new ArrayList());
    public static Map<String, List<SocketIOClient>> ALL_DP_CLIENT = PlatformDependent.newConcurrentHashMap();//用户名和大屏的映射
    public static Map<String, List<SocketIOClient>> ALL_CONTROL_CLIENT = PlatformDependent.newConcurrentHashMap();//用户名和控制器的映射
    public static List<SocketIOClient> ALL_CONTROL = Collections.synchronizedList(new ArrayList());

    public List<SocketIOClient> getCanControlDPByUserName(String userName) {
        return ALL_DP_CLIENT.get(userName);
    }

    private String getUserName(SocketIOClient client) {
        HandshakeData handshakeData = client.getHandshakeData();
        Map<String, List<String>> urlParams = handshakeData.getUrlParams();
        List<String> userName = urlParams.get("userName");
        if (userName != null) {
            if (!userName.isEmpty()) {
                return userName.get(0);
            }
        }
        return null;
    }

    private String getFrom(SocketIOClient client) {
        HandshakeData handshakeData = client.getHandshakeData();
        Map<String, List<String>> urlParams = handshakeData.getUrlParams();
        List<String> from = urlParams.get("from");
        if (from != null) {
            if (!from.isEmpty()) {
                return from.get(0);
            }
        }
        return null;
    }
    private String getIp(SocketIOClient client) {
        HandshakeData handshakeData = client.getHandshakeData();
        return handshakeData.getAddress().getAddress().getHostAddress();
    }

    SocketIOServer socketIOServer;

    @Override
    public void init(FilterConfig filterConfig) {
        super.init(filterConfig);
        socketIOServer = SocketIOUitls.getSocketIOServer();
        if (socketIOServer != null) {
            socketIOServer.getNamespace("/system").addEventListener("joinDP", String.class, (client, data, ackReq) -> {
                ALL_CLIENT.add(client);
                String userName = getUserName(client);
                if (userName != null) {
                    List<SocketIOClient> socketIOClients = ALL_DP_CLIENT.get(userName);
                    if (socketIOClients != null) {
                        socketIOClients.add(client);
                    } else {
                        List list = Collections.synchronizedList(new ArrayList());
                        list.add(client);
                        ALL_DP_CLIENT.put(userName, list);
                    }
                }
                String ip = getIp(client);
                FineLoggerFactory.getLogger().info("{} 用户 ip :{} 加入大屏列表 ", userName, ip);
                List<SocketIOClient> socketIOClients = ALL_CONTROL_CLIENT.get(userName);
                if (socketIOClients != null) {
                    for (SocketIOClient socketIOClient : socketIOClients) {
                        JSONObject entries = new JSONObject();
                        entries.put("ip", ip);
                        socketIOClient.sendEvent("dpJoin",entries);
                    }
                }
                FineLoggerFactory.getLogger().info("收到用户加入事件:{}", client);
            });
            socketIOServer.getNamespace("/system").addEventListener("leaveDP", String.class, (client, data, ackReq) -> {
                String userName = getUserName(client);
                String ip = getIp(client);
                ALL_CLIENT.remove(client);
                List<SocketIOClient> socketIOClients = ALL_CONTROL_CLIENT.get(userName);
                for (SocketIOClient socketIOClient : socketIOClients) {
                    JSONObject entries = new JSONObject();
                    entries.put("ip", ip);
                    socketIOClient.sendEvent("dpLeave",entries);
                }
                FineLoggerFactory.getLogger().info("离开大屏:{}", client);
            });
            socketIOServer.getNamespace("/system").addEventListener("joinControl", String.class, (client, data, ackReq) -> {
                ALL_CLIENT.remove(client);
                FineLoggerFactory.getLogger().info("收到用户加入事件:{}", client);
            });
            socketIOServer.getNamespace("/system").addDisconnectListener((client) -> {
                String userName = getUserName(client);
                String ip = getIp(client);
                FineLoggerFactory.getLogger().info("{} 用户 ip :{} 断开连接 ", userName, ip);
                ALL_CLIENT.remove(client);
                String from = getFrom(client);
                if (StringUtils.isNotBlank(from)) {
                    //判断是否来自大屏
                    if(StringUtils.equals("dp",from)){
                        List<SocketIOClient> socketIOClients = ALL_DP_CLIENT.get(userName);
                        if (socketIOClients != null) {
                            socketIOClients.remove(client);
                        }
                    }else{
                        //从控制端移除控制器
                        ALL_CONTROL.remove(client);
                    }
                }
            });
            socketIOServer.getNamespace("/system").addEventListener("clickMessage", String.class, (client, data, ackReq) -> {
                FineLoggerFactory.getLogger().info("收到点击事件:{}, data:{}", client, data);
                //这里不要用socketIOServer直接getBroadcastOperations 这样没有带namespace 要用下面的
                String userName = getUserName(client);
                List<SocketIOClient> socketIOClients = ALL_DP_CLIENT.get(userName);
                if (socketIOClients != null) {
                    for (SocketIOClient socketIOClient : socketIOClients) {
                        socketIOClient.sendEvent("clickMessage", data);
                    }
                }
//                SocketIOServerFactory.getBroadcastOperations().sendEvent("clickMessage",data);
            });
        }
    }

    @Override
    public void destroy() {
        super.destroy();
        if (socketIOServer != null) {
            socketIOServer.getNamespace("/system").removeAllListeners("clickMessage");
            socketIOServer.getNamespace("/system").removeAllListeners("joinDP");
        }
    }
}
